searchData={"items":[{"type":"module","title":"Jiken","doc":"`Jiken` is a library for dynamically mocking and unmocking functions in Elixir modules.\n\nIts main goal is to be used for simulation testing in development.\n\nIt provides an API to set up when and how different functions should be mocked, and then reset.\n\nThe goal is to be able to do something like the following: \n  - make a function predictably raise an exception at a certain time \n  - make a function predictably raise an exception after a certain amount of reductions (function calls)\n  - make a function raise an exception in a pseudo-random manner\n  - load and use production exceptions from AppSignal, Sentry, or Datadog\n\nThe purpose of `Jiken` is to be able to test critical functionality by easily simulating breaking scenarios.","ref":"Jiken.html"},{"type":"module","title":"Examples - Jiken","doc":"Consider the `Jiken.Dummy` module, which can be found here in the library itself.\nIt has the `greet/0` function.\n\n  ```\n  def greet do\n    IO.puts(\"Hello, world!\")\n  end\n  ```\n\nThis is how it can be dynamically mocked with `Jiken`.\n\n    iex> Jiken.Dummy()\n    \"Hello, world!\"\n    :ok \n\n    iex> Jiken.set(Jiken.Dummy, :greet, fn -> IO.puts(\"Hello, simulation!\") end)\n\n    iex> Jiken.Dummy()\n    Hello, simulation!\n    :ok\n\n    iex> Jiken.reset(Jiken.Dummy)\n    :ok\n\n    iex> Jiken.Dummy()\n    Hello, world!\n    :ok","ref":"Jiken.html#module-examples"},{"type":"function","title":"Jiken.init/0","doc":"","ref":"Jiken.html#init/0"},{"type":"function","title":"Jiken.mocked_modules_table/0","doc":"","ref":"Jiken.html#mocked_modules_table/0"},{"type":"function","title":"Jiken.original_modules_table/0","doc":"","ref":"Jiken.html#original_modules_table/0"},{"type":"function","title":"Jiken.reset/1","doc":"See  Jiken.Loaders.Unmock.reset/1 .","ref":"Jiken.html#reset/1"},{"type":"function","title":"Jiken.set/4","doc":"See  Jiken.Loaders.Mock.set/4 .","ref":"Jiken.html#set/4"},{"type":"module","title":"Jiken.Dummy","doc":"Used as a dummy module to test Jiken functionality.","ref":"Jiken.Dummy.html"},{"type":"function","title":"Jiken.Dummy.greet/0","doc":"","ref":"Jiken.Dummy.html#greet/0"},{"type":"function","title":"Jiken.Dummy.greet/1","doc":"","ref":"Jiken.Dummy.html#greet/1"},{"type":"module","title":"Jiken.Loaders.Mock","doc":"All functionality related to overwriting code is handled in this module.\n\nJiken first checks if the mocked function exists.\nIf it does, it stores the original module with a suffix to its namespace of .Original.\nIt is stored in ETS.\n\nJiken also stores the mocked implementation in a different ETS table.\nGlobally mocked functions and per-process mocks are stored separately.\n\nThen, Jiken goes through all of the function definitions of the module until it finds the mocked function,\nit updates its implementation to mocked version of the function from ETS,\ndepending on if it is mocked globally or per process.\n\nIf it cannot find a mock at this step, it falls back to the original implementation.","ref":"Jiken.Loaders.Mock.html"},{"type":"function","title":"Jiken.Loaders.Mock.set/4","doc":"","ref":"Jiken.Loaders.Mock.html#set/4"},{"type":"module","title":"Jiken.Loaders.Unmock","doc":"This module handles the reverting of a mocked module.\n\nCurrently, it reloads the whole original module.\n\nWe will make it reload only specific functions,\nso that we can dynamically mock different parts of modules.","ref":"Jiken.Loaders.Unmock.html"},{"type":"function","title":"Jiken.Loaders.Unmock.reset/1","doc":"","ref":"Jiken.Loaders.Unmock.html#reset/1"},{"type":"extras","title":"License","doc":"The MIT License (MIT)\nCopyright © 2025 Nikolay Dyulgerov\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"# Jiken\n\nThis library allows developers to simulate failures in their Mix applications. Different functions are configured to fail with specific errors, and revert to their normal operation after.\n\nIt is intended to be used in development, but eventually the goal is to support staging usage as well.","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"The package can be installed by adding `jiken` to your list of dependencies in `mix.exs`:\nMake sure to include it only in the `:dev` environment.\n\n```elixir\ndef deps do\n  [\n    {:jiken, \"~> 0.0.1\", only: [:dev]}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Overview","doc":"This is how a function can be dynamically mocked with `Jiken`.\n\n```elixir\niex> Jiken.Dummy()\n\"Hello, world!\"\n:ok \n\niex> Jiken.set(Jiken.Dummy, :greet, fn -> IO.puts(\"Hello, simulation!\") end)\n\niex> Jiken.Dummy()\nHello, simulation!\n:ok\n\niex> Jiken.reset(Jiken.Dummy)\n:ok\n\niex> Jiken.Dummy()\nHello, world!\n:ok\n```","ref":"readme.html#usage"},{"type":"extras","title":"Roadmap - Overview","doc":"- [ ] Simulation configuration:\n  - [ ] In a `.jiken.exs` file define which functions to be mocked\n  - [ ] Support different mocking strategies:\n    - [ ] Mock after a specific number of reductions (function calls)\n    - [ ] Mock at or after a specific time\n    - [ ] Specify when to unmock\n- [ ] Pull exceptions from production and simulate them locally:\n  - [ ] from AppSignal\n  - [ ] from Sentry\n  - [ ] from Datadog\n- [ ] Collect metrics for simulated exceptions\n- [ ] Support telemetry\n- [ ] Support more tailored usage in staging\n- [ ] Support for coordination across distributed nodes","ref":"readme.html#roadmap"},{"type":"extras","title":"Inspirations - Overview","doc":"The biggest inspirations for this library are the following:\n- [Mimic](https://github.com/edgurgel/mimic) - I've been using Mimic for a while now, and it's a great library.\nI want to better understand it and also to apply its approach to learn more about Elixir and OTP.\n- [sled simulation guide (jepsen-proof engineering)](https://sled.rs/simulation.html) - I've been reading more about distributed systems and stumbled upon this post.\nI can't hide that the topic is beyond me for now, but I'm interested in going deeper into it.","ref":"readme.html#inspirations"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2024 Nikolay Dyulgerov\n\nThis library is [MIT licensed](https://github.com/nicolayd/jiklen/blob/main/LICENSE.md). See the LICENSE.md for details.","ref":"readme.html#copyright-and-license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.0"}}